==> download.php?FileName=terralib-5.2.0.zip: src/terralib/qt/widgets/rp/ClassifierWizard.cpp <==
/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/rp/ClassifierWizard.cpp

  \brief A Qt dialog that allows users to run a classifier operation defined by RP module.
*/

// TerraLib 
#include "../../../common/progress/ProgressManager.h"
#include "../../../dataaccess/dataset/DataSet.h"
#include "../../../dataaccess/utils/Utils.h"
#include "../../../raster/Grid.h"
#include "../../../raster/Raster.h"
#include "../../../rp/Classifier.h"
#include "../../../rp/Module.h"
#include "../../../se/Categorize.h"
#include "../../../se/ColorMap.h"
#include "../../../se/ParameterValue.h"
#include "../../../se/RasterSymbolizer.h"
#include "../../../se/Utils.h"
#include "../../../maptools/Utils.h"
#include "../help/HelpPushButton.h"
#include "../layer/search/LayerSearchWidget.h"
#include "../layer/search/LayerSearchWizardPage.h"
#include "../progress/ProgressViewerDialog.h"
#include "ClassifierWizard.h"
#include "ClassifierWizardPage.h"
#include "../raster/RasterInfoWidget.h"
#include "RasterInfoWizardPage.h"
#include "Utils.h"

// STL
#include <cassert>

// Qt
#include <QMessageBox>
#include <QApplication>
#include <QActionGroup>


te::qt::widgets::ClassifierWizard::ClassifierWizard(QWidget* parent)
  : QWizard(parent)
{
  //configure the wizard
  this->setWizardStyle(QWizard::ModernStyle);
  this->setWindowTitle(tr("Classifier"));
  //this->setFixedSize(640, 480);

  this->setOption(QWizard::HaveHelpButton, true);
  this->setOption(QWizard::HelpButtonOnRight, false);

  te::qt::widgets::HelpPushButton* helpButton = new te::qt::widgets::HelpPushButton(this);

  this->setButton(QWizard::HelpButton, helpButton);

  helpButton->setPageReference("plugins/rp/rp_classifier.html");

  addPages();
}

te::qt::widgets::ClassifierWizard::~ClassifierWizard()
{
  m_classifierPage->clearCanvas();
}

bool te::qt::widgets::ClassifierWizard::validateCurrentPage()
{
  if(currentPage() ==  m_layerSearchPage.get())
  {
    std::list<te::map::AbstractLayerPtr> list = m_layerSearchPage->getSearchWidget()->getSelecteds();

    if(list.empty() == false)
    {
      te::map::AbstractLayerPtr l = *list.begin();

      m_classifierPage->set(l);
    }

    return m_layerSearchPage->isComplete();
  }
  else if(currentPage() ==  m_classifierPage.get())
  {
    return m_classifierPage->isComplete();
  }
  else if(currentPage() ==  m_rasterInfoPage.get())
  {
    return execute();
  }

  return true;
}

void te::qt::widgets::ClassifierWizard::setList(std::list<te::map::AbstractLayerPtr>& layerList)
{
  m_layerSearchPage->getSearchWidget()->setList(layerList);
  m_layerSearchPage->getSearchWidget()->filterOnlyByRaster();

  m_classifierPage->setList(layerList);
}

void te::qt::widgets::ClassifierWizard::setMapDisplay(te::qt::widgets::MapDisplay* mapDisplay)
{
  m_classifierPage->setMapDisplay(mapDisplay);
}

void te::qt::widgets::ClassifierWizard::setActionGroup(QActionGroup* actionGroup)
{
  m_classifierPage->setActionGroup(actionGroup);
}

te::map::AbstractLayerPtr te::qt::widgets::ClassifierWizard::getOutputLayer()
{
  return m_outputLayer;
}

void te::qt::widgets::ClassifierWizard::addPages()
{
  m_layerSearchPage.reset(new te::qt::widgets::LayerSearchWizardPage(this));
  m_classifierPage.reset(new te::qt::widgets::ClassifierWizardPage(this));
  m_rasterInfoPage.reset(new te::qt::widgets::RasterInfoWizardPage(this));

  addPage(m_layerSearchPage.get());
  addPage(m_classifierPage.get());
  addPage(m_rasterInfoPage.get());

  //for contrast only one layer can be selected
  m_layerSearchPage->getSearchWidget()->enableMultiSelection(false);
}

bool te::qt::widgets::ClassifierWizard::execute()
{
  //get layer
  std::list<te::map::AbstractLayerPtr> list = m_layerSearchPage->getSearchWidget()->getSelecteds();
  te::map::AbstractLayerPtr l = *list.begin();

  std::auto_ptr<te::rst::Raster> inputRst(te::map::GetRaster(l.get()));

  //run contrast
  te::rp::Classifier algorithmInstance;

  te::rp::Classifier::InputParameters algoInputParams = m_classifierPage->getInputParams();
  algoInputParams.m_inputRasterPtr = inputRst.get();

  te::rp::Classifier::OutputParameters algoOutputParams = m_classifierPage->getOutputParams();
  algoOutputParams.m_rInfo = m_rasterInfoPage->getWidget()->getInfo();
  algoOutputParams.m_rType = m_rasterInfoPage->getWidget()->getType();

  //progress
  te::qt::widgets::ProgressViewerDialog v(this);
  int id = te::common::ProgressManager::getInstance().addViewer(&v);

  QApplication::setOverrideCursor(Qt::WaitCursor);

  if(algorithmInstance.initialize(algoInputParams))
  {
    if(algorithmInstance.execute(algoOutputParams))
    {
      algoOutputParams.reset();

      //set output layer
      m_outputLayer = te::qt::widgets::createLayer(m_rasterInfoPage->getWidget()->getType(), 
                                                   m_rasterInfoPage->getWidget()->getInfo());

      int addOutPutLayer = QMessageBox::question(this, tr("Classifier"), tr("Would you like to add layer on active project?"),
                                                 QMessageBox::Yes | QMessageBox::No);

      if(addOutPutLayer == QMessageBox::Yes)
          emit addLayer(m_outputLayer);

      QMessageBox::information(this, tr("Classifier"), tr("Classifier ended sucessfully."));
    }
    else
    {
      QMessageBox::critical(this, tr("Classifier"), tr("Classifier execution error.") +
        ( " " + te::rp::Module::getLastLogStr() ).c_str());

      te::common::ProgressManager::getInstance().removeViewer(id);

      QApplication::restoreOverrideCursor();

      return false;
    }
  }
  else
  {
    QMessageBox::critical(this, tr("Classifier"), tr("Classifier initialization error.") +
      ( " " + te::rp::Module::getLastLogStr() ).c_str() );

    te::common::ProgressManager::getInstance().removeViewer(id);

    QApplication::restoreOverrideCursor();

    return false;
  }

  //create legend
  te::cl::ROISet* rs = m_classifierPage->getROISet();

  if(rs && rs->getROISet().empty() == false)
  {
    te::se::ColorMap* cm = new te::se::ColorMap();

    te::se::Categorize* c = new te::se::Categorize();

    c->setFallbackValue("#000000");
    c->setLookupValue(new te::se::ParameterValue("Rasterdata"));

    QColor cWhite(Qt::white);
    std::string colorWhiteStr = cWhite.name().toUtf8().data();

    //added dummy color for values < than min values...
    c->addValue(new te::se::ParameterValue(colorWhiteStr));

    std::map<std::string, te::cl::ROI*> roiSetMap = rs->getROISet();
    std::map<std::string, te::cl::ROI*>::iterator it = roiSetMap.begin();

    int count = 1;

    while(it != roiSetMap.end())
    {
      std::string color = it->second->getColor();
      std::string range = QString::number(count).toUtf8().data();
      

      c->addThreshold(new te::se::ParameterValue(range));
      c->addValue(new te::se::ParameterValue(color));

      if(count == roiSetMap.size())
      {
        std::string rangeNext = QString::number(count + 1).toUtf8().data();
        c->addThreshold(new te::se::ParameterValue(rangeNext));
      }

      ++count;
      ++it;
    }

    //added dummy color for values > than max values...
    c->addValue(new te::se::ParameterValue(colorWhiteStr));

    c->setThresholdsBelongTo(te::se::Categorize::SUCCEEDING);
    cm->setCategorize(c);

    te::se::RasterSymbolizer* rasterSymb = te::se::GetRasterSymbolizer(m_outputLayer->getStyle());
    rasterSymb->setColorMap(cm);
  }

  te::common::ProgressManager::getInstance().removeViewer(id);

  QApplication::restoreOverrideCursor();

  return true;
}

==> download.php?FileName=terralib-5.3.2.zip: src/terralib/qt/widgets/rp/ClassifierWizard.cpp <==
/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/rp/ClassifierWizard.cpp

  \brief A Qt dialog that allows users to run a classifier operation defined by RP module.
*/

// TerraLib
#include "../../../common/progress/ProgressManager.h"
#include "../../../color/ColorBar.h"
#include "../../../color/ColorScheme.h"
#include "../../../color/ColorSchemeCatalog.h"
#include "../../../color/ColorSchemeCatalogManager.h"
#include "../../../color/ColorSchemeGroup.h"
#include "../../../color/RGBAColor.h"
#include "../../../dataaccess/dataset/DataSet.h"
#include "../../../dataaccess/utils/Utils.h"
#include "../../../raster/Grid.h"
#include "../../../raster/Raster.h"
#include "../../../raster/RasterSummary.h"
#include "../../../raster/RasterSummaryManager.h"
#include "../../../rp/Classifier.h"
#include "../../../rp/Module.h"
#include "../../../se/Categorize.h"
#include "../../../se/ColorMap.h"
#include "../../../se/ParameterValue.h"
#include "../../../se/RasterSymbolizer.h"
#include "../../../se/Utils.h"
#include "../../../maptools/Utils.h"
#include "../../../maptools/GroupingAlgorithms.h"
#include "../help/HelpPushButton.h"
#include "../layer/search/LayerSearchWidget.h"
#include "../layer/search/LayerSearchWizardPage.h"
#include "../progress/ProgressViewerDialog.h"
#include "ClassifierWizard.h"
#include "ClassifierWizardPage.h"
#include "../raster/RasterInfoWidget.h"
#include "RasterInfoWizardPage.h"
#include "Utils.h"

// STL
#include <cassert>

// Qt
#include <QMessageBox>
#include <QApplication>
#include <QActionGroup>

te::qt::widgets::ClassifierWizard::ClassifierWizard(QWidget* parent)
  : QWizard(parent)
{
  //configure the wizard
  this->setWizardStyle(QWizard::ModernStyle);
  this->setWindowTitle(tr("Classifier"));
  //this->setFixedSize(640, 480);

  this->setOption(QWizard::HaveHelpButton, true);
  this->setOption(QWizard::HelpButtonOnRight, false);

  te::qt::widgets::HelpPushButton* helpButton = new te::qt::widgets::HelpPushButton(this);

  this->setButton(QWizard::HelpButton, helpButton);

  helpButton->setPageReference("plugins/rp/rp_classifier.html");

  connect(this, SIGNAL(currentIdChanged(int)), SLOT(onPageChanged(int)));

  addPages();
}

te::qt::widgets::ClassifierWizard::~ClassifierWizard()
{
  m_classifierPage->clearCanvas();
}

bool te::qt::widgets::ClassifierWizard::validateCurrentPage()
{
  if(currentPage() ==  m_layerSearchPage.get())
  {
    std::list<te::map::AbstractLayerPtr> list = m_layerSearchPage->getSearchWidget()->getSelecteds();

    if(list.empty() == false)
    {
      te::map::AbstractLayerPtr l = *list.begin();

      m_classifierPage->set(l);
    }

    return m_layerSearchPage->isComplete();
  }
  else if(currentPage() ==  m_classifierPage.get())
  {
    return m_classifierPage->isComplete();
  }
  else if(currentPage() ==  m_rasterInfoPage.get())
  {
    return execute();
  }

  return true;
}

void te::qt::widgets::ClassifierWizard::setList(std::list<te::map::AbstractLayerPtr>& layerList)
{
  m_layerSearchPage->getSearchWidget()->setList(layerList);
  m_layerSearchPage->getSearchWidget()->filterOnlyByRaster();

  m_classifierPage->setList(layerList);
}

void te::qt::widgets::ClassifierWizard::setMapDisplay(te::qt::widgets::MapDisplay* mapDisplay)
{
  m_classifierPage->setMapDisplay(mapDisplay);
}

void te::qt::widgets::ClassifierWizard::setActionGroup(QActionGroup* actionGroup)
{
  m_classifierPage->setActionGroup(actionGroup);
}

te::map::AbstractLayerPtr te::qt::widgets::ClassifierWizard::getOutputLayer()
{
  return m_outputLayer;
}

void te::qt::widgets::ClassifierWizard::onMapDisplayExtentChanged()
{
  m_classifierPage->onMapDisplayExtentChanged();
}

void te::qt::widgets::ClassifierWizard::addPages()
{
  m_layerSearchPage.reset(new te::qt::widgets::LayerSearchWizardPage(this));
  m_classifierPage.reset(new te::qt::widgets::ClassifierWizardPage(this));
  m_rasterInfoPage.reset(new te::qt::widgets::RasterInfoWizardPage(this));

  addPage(m_layerSearchPage.get());
  addPage(m_classifierPage.get());
  addPage(m_rasterInfoPage.get());

  //for contrast only one layer can be selected
  m_layerSearchPage->getSearchWidget()->enableMultiSelection(false);
}

bool te::qt::widgets::ClassifierWizard::execute()
{
  if(m_rasterInfoPage->getWidget()->fileExists())
  {
    QMessageBox::warning(this, tr("Classifier"), tr("File already exists."));
    return false;
  }

  //get layer
  std::list<te::map::AbstractLayerPtr> list = m_layerSearchPage->getSearchWidget()->getSelecteds();
  te::map::AbstractLayerPtr l = *list.begin();

  std::unique_ptr<te::rst::Raster> inputRst(te::map::GetRaster(l.get()));

  //run contrast
  te::rp::Classifier algorithmInstance;

  te::rp::Classifier::InputParameters algoInputParams = m_classifierPage->getInputParams();
  algoInputParams.m_inputRasterPtr = inputRst.get();

  if(
      ( m_classifierPage->getROISet() != 0 )
      &&
      ( ! m_classifierPage->getROISet()->getROISet().empty() )
    )
  {
    algoInputParams.m_createRasterPalette = false;
  }
  else
  {
    algoInputParams.m_createRasterPalette = m_classifierPage->getPallete();
  }

  te::rp::Classifier::OutputParameters algoOutputParams = m_classifierPage->getOutputParams();
  algoOutputParams.m_rInfo = m_rasterInfoPage->getWidget()->getInfo();
  algoOutputParams.m_rType = m_rasterInfoPage->getWidget()->getType();

  //progress
  te::qt::widgets::ProgressViewerDialog v(this);

  QApplication::setOverrideCursor(Qt::WaitCursor);

  if(algorithmInstance.initialize(algoInputParams))
  {
    if(algorithmInstance.execute(algoOutputParams))
    {
      algoOutputParams.reset();

      //set output layer
      m_outputLayer = te::qt::widgets::createLayer(m_rasterInfoPage->getWidget()->getType(), 
                                                   m_rasterInfoPage->getWidget()->getInfo());

      QMessageBox::information(this, tr("Classifier"), tr("Classifier ended sucessfully."));
    }
    else
    {
      QMessageBox::critical(this, tr("Classifier"), tr("Classifier execution error.") +
        ( " " + te::rp::Module::getLastLogStr() ).c_str());

      QApplication::restoreOverrideCursor();

      return false;
    }
  }
  else
  {
    QMessageBox::critical(this, tr("Classifier"), tr("Classifier initialization error.") +
      ( " " + te::rp::Module::getLastLogStr() ).c_str() );

    QApplication::restoreOverrideCursor();

    return false;
  }

  //create legend
  te::cl::ROISet* rs = m_classifierPage->getROISet();

  if(rs && rs->getROISet().empty() == false)
  {
    te::se::ColorMap* cm = new te::se::ColorMap();

    te::se::Categorize* c = new te::se::Categorize();

    c->setFallbackValue("#000000");
    c->setLookupValue(new te::se::ParameterValue("Rasterdata"));

    QColor cWhite(Qt::white);
    std::string colorWhiteStr = cWhite.name().toUtf8().data();

    //added dummy color for values < than min values...
    c->addValue(new te::se::ParameterValue(colorWhiteStr));

    std::map<std::string, te::cl::ROI*> roiSetMap = rs->getROISet();
    std::map<std::string, te::cl::ROI*>::iterator it = roiSetMap.begin();

    unsigned int count = 1;

    while(it != roiSetMap.end())
    {
      std::string color = it->second->getColor();
      std::string range = QString::number(count).toUtf8().data();
      

      c->addThreshold(new te::se::ParameterValue(range));
      c->addValue(new te::se::ParameterValue(color));

      if(count == roiSetMap.size())
      {
        std::string rangeNext = QString::number(count + 1).toUtf8().data();
        c->addThreshold(new te::se::ParameterValue(rangeNext));
      }

      ++count;
      ++it;
    }

    //added dummy color for values > than max values...
    c->addValue(new te::se::ParameterValue(colorWhiteStr));

    c->setThresholdsBelongTo(te::se::Categorize::SUCCEEDING);
    cm->setCategorize(c);

    te::se::RasterSymbolizer* rasterSymb = te::se::GetRasterSymbolizer(m_outputLayer->getStyle());
    rasterSymb->setColorMap(cm);
  }
//   else
//   {
//     buildLegend();
//   }

  QApplication::restoreOverrideCursor();

  te::qt::widgets::applyRasterMultiResolution(tr("Classifier"), te::map::GetRaster(m_outputLayer.get()));

  emit addLayer(m_outputLayer);

  return true;
}

void te::qt::widgets::ClassifierWizard::buildLegend()
{
  //Calc min and max
  const te::rst::RasterSummary* rsMin = te::rst::RasterSummaryManager::getInstance().get(te::map::GetRaster(m_outputLayer.get()), te::rst::SUMMARY_MIN);
  const te::rst::RasterSummary* rsMax = te::rst::RasterSummaryManager::getInstance().get(te::map::GetRaster(m_outputLayer.get()), te::rst::SUMMARY_MAX);
  const std::complex<double>* cmin = rsMin->at(0).m_minVal;
  const std::complex<double>* cmax = rsMax->at(0).m_maxVal;
  double min = cmin->real();
  double max = cmax->real();

  std::vector<double> vec;
  vec.push_back(min);
  vec.push_back(max);

  //Get color
  std::vector<te::color::RGBAColor> colorVec;

  te::color::ColorSchemeCatalog* csc = te::color::ColorSchemeCatalogManager::getInstance().findByName("Default");

  if(!csc)
    return;

  std::vector<te::color::ColorSchemeGroup*> csgVec = csc->getColorSchemeGroups();

  for(std::size_t t = 0; t < csgVec.size(); ++t)
  {
    te::color::ColorSchemeGroup* csg = csgVec[t];

    if(csg->getName() == "Basic")
    {
      std::vector<te::color::ColorScheme*> csVec = csg->getColorSchemes();

      for(std::size_t p = 0; p < csVec.size(); ++p)
      {
        te::color::ColorScheme* cs = csVec[p];

        if(cs->getName() == "Red")
        {
          std::vector<te::color::RGBAColor>* colors = cs->getColors()[0];     //WARNING... Always getting the first position

          std::vector<te::color::RGBAColor>::iterator it = colors->begin();

          // create color bar
          te::color::ColorBar* cb = new te::color::ColorBar(*(colors->begin()), *(colors->end() - 1), 256);

          int count = 0;

          //fill color bar
          while(it != colors->end())
          {
            if(count != 0 && count != static_cast<int>(colors->size()) - 1)
            {
              double pos = (1. / (colors->size() - 1)) * count;

              cb->addColor(*it, pos);
            }

            ++count;
            ++it;
          }
          colorVec = cb->getSlices(2);
        }
      }
    }
  }

  //Get legend
  std::vector<te::map::GroupingItem*> legVec;

  te::map::GroupingByEqualSteps(vec.begin(), vec.end(), 2, legVec);

  te::se::Categorize* c = new te::se::Categorize();

  c->setFallbackValue("#000000");
  c->setLookupValue(new te::se::ParameterValue("Rasterdata"));

  QColor cWhite(Qt::white);
  std::string colorWhiteStr = cWhite.name().toUtf8().data();

  //added dummy color for values < than min values...
  c->addValue(new te::se::ParameterValue(colorWhiteStr));

  for(std::size_t i = 0; i < legVec.size(); ++i)
  {
    QColor color(colorVec[i].getRed(), colorVec[i].getGreen(), colorVec[i].getBlue(), colorVec[i].getAlpha());

    std::string rangeStr = legVec[i]->getLowerLimit();
    std::string colorStr = color.name().toUtf8().data();

    c->addThreshold(new te::se::ParameterValue(rangeStr));
    c->addValue(new te::se::ParameterValue(colorStr));

    if(legVec[i] == legVec.back())
    {
      rangeStr = legVec[i]->getUpperLimit();
      c->addThreshold(new te::se::ParameterValue(rangeStr));
    }
  }

  c->addValue(new te::se::ParameterValue(colorWhiteStr));

  c->setThresholdsBelongTo(te::se::Categorize::SUCCEEDING);

  te::se::ColorMap* cm = new te::se::ColorMap();

  cm->setCategorize(c);
  cm->setInterpolate(0);
  cm->setRecode(0);

  te::se::RasterSymbolizer* rasterSymb = te::se::GetRasterSymbolizer(m_outputLayer->getStyle());
  rasterSymb->setColorMap(cm);
}

void te::qt::widgets::ClassifierWizard::onPageChanged(int index)
{
  this->adjustSize();
}
